# -*- coding: utf-8 -*-
"""最大正向匹配分词"""
from pypinyin.constants import PHRASES_DICT


class Seg(object):
    """正向最大匹配分词

    :type prefix_set: PrefixSet
    :param no_non_phrases: 是否严格按照词语分词，不允许把非词语的词当做词语进行分词
    :type no_non_phrases: bool
    """

    def __init__(self, prefix_set, no_non_phrases=False):
        self._prefix_set = prefix_set
        self._no_non_phrases = no_non_phrases

    def cut(self, text):
        """分词

        :param text: 待分词的文本
        :yield: 单个词语
        """
        remain = text
        while remain:
            matched = ''
            last_valid_word = ''
            last_valid_index = 0

            # 一次加一个字的匹配
            for index in range(len(remain)):
                word = remain[:index + 1]
                if word in self._prefix_set:
                    matched = word
                    # 检查当前匹配的词是否为有效词语
                    if (not self._no_non_phrases) or word in PHRASES_DICT:
                        last_valid_word = word
                        last_valid_index = index + 1
                else:
                    # 前缀匹配失败，需要处理之前的匹配结果
                    if last_valid_word:
                        # 有有效词语，输出最后一个有效词语
                        yield last_valid_word
                        remain = remain[last_valid_index:]
                    else:
                        # 没有有效词语
                        if self._no_non_phrases:
                            # 严格模式：输出第一个字符
                            yield remain[0]
                            remain = remain[1:]
                        else:
                            # 非严格模式：输出匹配到的前缀（如果有）或第一个字符
                            if matched:
                                yield matched
                                remain = remain[len(matched):]
                            else:
                                yield remain[0]
                                remain = remain[1:]
                    break
            else:  # 整个剩余文本都能匹配前缀
                if last_valid_word:
                    # 有有效词语，输出最后一个有效词语
                    yield last_valid_word
                    remain = remain[last_valid_index:]
                else:
                    # 没有有效词语，处理剩余文本
                    if self._no_non_phrases and remain not in PHRASES_DICT:
                        # 严格模式且不在词典中：拆分为单字符
                        for x in remain:
                            yield x
                    else:
                        # 非严格模式或在词典中：输出整个剩余文本
                        yield remain
                    break

    def train(self, words):
        """训练分词器

        :param words: 词语列表
        """
        self._prefix_set.train(words)


class PrefixSet(object):
    def __init__(self):
        self._set = set()

    def train(self, word_s):
        """更新 prefix set

        :param word_s: 词语库列表
        :type word_s: iterable
        :return: None
        """
        for word in word_s:
            # 把词语的每个前缀更新到 prefix_set 中
            for index in range(len(word)):
                self._set.add(word[:index + 1])

    def __contains__(self, key):
        return key in self._set


p_set = PrefixSet()
p_set.train(PHRASES_DICT.keys())

#: 基于内置词库的最大正向匹配分词器。使用:
#:
#: .. code-block:: python
#:
#:     >>> from pypinyin.contrib.mmseg import seg
#:     >>> text = '你好，我是中国人，我爱我的祖国'
#:     >>> seg.cut(text)
#:     <generator object Seg.cut at 0x10b2df2b0>
#:     >>> list(seg.cut(text))
#:     ['你好', '，', '我', '是', '中国人', '，', '我', '爱',
#:      '我的', '祖', '国']
#:     >>> seg.train(['祖国', '我是'])
#:     >>> list(seg.cut(text))
#:     ['你好', '，', '我是', '中国人', '，', '我', '爱',
#:      '我的', '祖国']
#:     >>>
seg = Seg(p_set, no_non_phrases=True)


def retrain(seg_instance):
    """重新使用内置词典训练 seg_instance。

    比如在增加自定义词语信息后需要调用这个模块重新训练分词器

    :type seg_instance: Seg
    """
    seg_instance.train(PHRASES_DICT.keys())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pytest import mark

from pypinyin import pinyin, Style, lazy_pinyin, slug
from pypinyin.seg.simpleseg import simple_seg


def test_import_all():
    pinyin('啦啦啦')         # noqa
    pinyin('啦啦啦', Style.TONE2)  # noqa
    lazy_pinyin('啦啦啦')    # noqa
    slug('啦啦啦')           # noqa


def test_simple_seg():
    assert simple_seg('啦啦') == ['啦啦']
    assert simple_seg('啦啦abc') == ['啦啦', 'abc']
    assert simple_seg('&##啦啦abc') == ['&##', '啦啦', 'abc']
    assert simple_seg('&#哦#啦啦abc') == ['&#', '哦', '#', '啦啦', 'abc']
    assert simple_seg('哦ほ#') == ['哦', 'ほ#']
    assert simple_seg(['啦啦']) == ['啦啦']
    assert simple_seg(['啦啦', 'abc']) == ['啦啦', 'abc']
    assert simple_seg('哦ほ#哪') == ['哦', 'ほ#', '哪']
    assert simple_seg('哦ほ#哪#') == ['哦', 'ほ#', '哪', '#']
    assert simple_seg('你好啊 --') == ['你好啊', ' --']
    assert simple_seg('啊 -- ') == ['啊', ' -- ']
    assert simple_seg('你好啊 -- 那') == ['你好啊', ' -- ', '那']
    assert simple_seg('啊 -- 你好那 ') == ['啊', ' -- ', '你好那', ' ']
    assert simple_seg('a 你好啊 -- 那 ') == ['a ', '你好啊', ' -- ', '那', ' ']
    assert simple_seg('a啊 -- 你好那 ') == ['a', '啊', ' -- ', '你好那', ' ']


def test_issue_205():
    assert pinyin('金融行业', Style.FIRST_LETTER)[2] == ['h']
    assert pinyin('军工行业', Style.FIRST_LETTER)[2] == ['h']
    assert pinyin('浦发银行', Style.FIRST_LETTER)[3] == ['h']
    assert pinyin('交通银行', Style.FIRST_LETTER)[3] == ['h']


def test_issue_266():
    assert lazy_pinyin('哟', style=Style.FINALS) == ['o']
    assert lazy_pinyin('哟', style=Style.FINALS, strict=False) == ['o']

    assert lazy_pinyin('嗯', style=Style.FINALS) == ['']
    assert lazy_pinyin('嗯', style=Style.FINALS, strict=False) == ['n']

    assert lazy_pinyin('呣', style=Style.FINALS) == ['']
    assert lazy_pinyin('呣', style=Style.FINALS, strict=False) == ['m']


def test_issue_311():
    assert lazy_pinyin('大城', style=Style.TONE2) == ['da4', 'che2ng']
    assert lazy_pinyin('大城市', style=Style.TONE2) == ['da4', 'che2ng', 'shi4']
    assert lazy_pinyin('大城区', style=Style.TONE2) == ['da4', 'che2ng', 'qu1']


@mark.parametrize('neutral_tone_with_five', [True, False])
def test_issue_284(neutral_tone_with_five):
    assert lazy_pinyin('嗯', style=Style.FINALS_TONE3,
                       neutral_tone_with_five=neutral_tone_with_five) == ['']
    assert lazy_pinyin('呣', style=Style.FINALS_TONE3,
                       neutral_tone_with_five=neutral_tone_with_five) == ['']
    assert lazy_pinyin('嗯', style=Style.FINALS,
                       neutral_tone_with_five=neutral_tone_with_five) == ['']
    assert lazy_pinyin('呣', style=Style.FINALS,
                       neutral_tone_with_five=neutral_tone_with_five) == ['']
    assert lazy_pinyin('嗯', style=Style.INITIALS,
                       neutral_tone_with_five=neutral_tone_with_five) == ['']
    assert lazy_pinyin('呣', style=Style.INITIALS,
                       neutral_tone_with_five=neutral_tone_with_five) == ['']


if __name__ == '__main__':
    import pytest
    pytest.cmdline.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class PinyinNotFoundException(Exception):
    #: 异常信息
    message = ''
    #: 不包含拼音的字符串
    chars = ''

    def __init__(self, chars):
        self.message = 'No pinyin found for character "{}"'.format(chars)
        self.chars = chars
        super(PinyinNotFoundException, self).__init__(self.message)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import pytest

from pypinyin import (
    pinyin, slug, lazy_pinyin, load_single_dict,
    load_phrases_dict, NORMAL, TONE, TONE2, TONE3, INITIALS,
    FIRST_LETTER, FINALS, FINALS_TONE, FINALS_TONE2, FINALS_TONE3,
    BOPOMOFO, BOPOMOFO_FIRST, CYRILLIC, CYRILLIC_FIRST, Style
)
from pypinyin.compat import SUPPORT_UCS4
from pypinyin.core import seg
from pypinyin.exceptions import PinyinNotFoundException


def test_pinyin_initials():
    """包含声明和韵母的词语"""
    hans = '中心'
    # 默认风格，带声调
    assert pinyin(hans) == [['zh\u014dng'], ['x\u012bn']]
    assert pinyin(hans, strict=False) == [['zh\u014dng'], ['x\u012bn']]
    # 普通风格，不带声调
    assert pinyin(hans, NORMAL) == [['zhong'], ['xin']]
    assert pinyin(hans, NORMAL, strict=False) == [['zhong'], ['xin']]
    # 声调风格，拼音声调在韵母第一个字母上
    assert pinyin(hans, TONE) == [['zh\u014dng'], ['x\u012bn']]
    assert pinyin(hans, TONE, strict=False) == [['zh\u014dng'], ['x\u012bn']]
    # 声调风格2，即拼音声调在各个声母之后，用数字 [1-4] 进行表示
    assert pinyin(hans, TONE2) == [['zho1ng'], ['xi1n']]
    assert pinyin(hans, TONE2, strict=False) == [['zho1ng'], ['xi1n']]
    # 声调风格3，即拼音声调在各个拼音之后，用数字 [1-4] 进行表示
    assert pinyin(hans, TONE3) == [['zhong1'], ['xin1']]
    assert pinyin(hans, TONE3, strict=False) == [['zhong1'], ['xin1']]
    # 声母风格，只返回各个拼音的声母部分
    assert pinyin(hans, INITIALS) == [['zh'], ['x']]
    assert pinyin(hans, INITIALS, strict=False) == [['zh'], ['x']]
    # 首字母风格，只返回拼音的首字母部分
    assert pinyin(hans, FIRST_LETTER) == [['z'], ['x']]
    assert pinyin(hans, FIRST_LETTER, strict=False) == [['z'], ['x']]
    # 注音风格，带声调
    assert pinyin(hans, BOPOMOFO) == [['ㄓㄨㄥ'], ['ㄒㄧㄣ']]
    assert pinyin(hans, BOPOMOFO, strict=False) == [['ㄓㄨㄥ'], ['ㄒㄧㄣ']]
    # 注音风格，首字母
    assert pinyin(hans, BOPOMOFO_FIRST) == [['ㄓ'], ['ㄒ']]
    assert pinyin(hans, BOPOMOFO_FIRST, strict=False) == [['ㄓ'], ['ㄒ']]
    # test CYRILLIC style
    assert pinyin(hans, CYRILLIC) == [['чжун1'], ['синь1']]
    assert pinyin(hans, CYRILLIC, strict=False) == [['чжун1'], ['синь1']]
    # CYRILLIC_FIRST style return only first letters
    assert pinyin(hans, CYRILLIC_FIRST) == [['ч'], ['с']]
    assert pinyin(hans, CYRILLIC_FIRST, strict=False) == [['ч'], ['с']]
    # 启用多音字模式
    assert pinyin(hans, heteronym=True) == [['zh\u014dng', 'zh\xf2ng'],
                                            ['x\u012bn']]
    assert pinyin(hans, heteronym=True, strict=False) == \
        [['zh\u014dng', 'zh\xf2ng'], ['x\u012bn']]
    # 韵母风格1，只返回各个拼音的韵母部分，不带声调
    assert pinyin(hans, style=FINALS) == [['ong'], ['in']]
    assert pinyin(hans, style=FINALS, strict=False) == [['ong'], ['in']]
    # 韵母风格2，带声调，声调在韵母第一个字母上
    assert pinyin(hans, style=FINALS_TONE) == [['\u014dng'], ['\u012bn']]
    assert pinyin(hans, style=FINALS_TONE, strict=False) == \
        [['\u014dng'], ['\u012bn']]
    # 韵母风格2，带声调，声调在各个声母之后，用数字 [1-4] 进行表示
    assert pinyin(hans, style=FINALS_TONE2) == [['o1ng'], ['i1n']]
    assert pinyin(hans, style=FINALS_TONE2, strict=False) == \
        [['o1ng'], ['i1n']]
    # 韵母风格3，带声调，声调在各个拼音之后，用数字 [1-4] 进行表示
    assert pinyin(hans, style=FINALS_TONE3) == [['ong1'], ['in1']]
    assert pinyin(hans, style=FINALS_TONE3, strict=False) == \
        [['ong1'], ['in1']]


def test_pinyin_finals():
    """只包含韵母的词语"""
    hans = '嗷嗷'
    assert pinyin(hans) == [['\xe1o'], ['\xe1o']]
    assert pinyin(hans + 'abc') == [['\xe1o'], ['\xe1o'], ['abc']]
    assert pinyin(hans, NORMAL) == [['ao'], ['ao']]
    assert pinyin(hans, TONE) == [['\xe1o'], ['\xe1o']]
    assert pinyin(hans, TONE2) == [['a2o'], ['a2o']]
    assert pinyin(hans, TONE3) == [['ao2'], ['ao2']]
    assert pinyin(hans, INITIALS) == [[''], ['']]
    assert pinyin(hans, FIRST_LETTER) == [['a'], ['a']]
    assert pinyin(hans, BOPOMOFO) == [['ㄠˊ'], ['ㄠˊ']]
    assert pinyin(hans, BOPOMOFO_FIRST) == [['ㄠ'], ['ㄠ']]
    assert pinyin(hans, CYRILLIC) == [['ао2'], ['ао2']]
    assert pinyin(hans, CYRILLIC_FIRST) == [['а'], ['а']]
    assert pinyin(hans, heteronym=True) == [['\xe1o'], ['\xe1o']]
    assert pinyin('啊', heteronym=True) == \
        [['a', 'ā', 'á', 'ǎ', 'à', 'è']]
    assert pinyin(hans, style=FINALS) == [['ao'], ['ao']]
    assert pinyin(hans, style=FINALS_TONE) == [['\xe1o'], ['\xe1o']]
    assert pinyin(hans, style=FINALS_TONE2) == [['a2o'], ['a2o']]
    assert pinyin(hans, style=FINALS_TONE3) == [['ao2'], ['ao2']]


def test_slug():
    hans = '中心'
    assert slug(hans) == 'zhong-xin'
    assert slug(hans, heteronym=True) == 'zhong-xin'


def test_zh_and_en():
    """中英文混合的情况"""
    # 中英文
    hans = '中心'
    assert pinyin(hans + 'abc') == [['zh\u014dng'], ['x\u012bn'], ['abc']]
    # 中英文混合的固定词组
    assert pinyin('黄山B股', style=TONE2) == \
        [['hua2ng'], ['sha1n'], ['B'], ['gu3']]
    assert pinyin('A股', style=TONE2) == [['A'], ['gu3']]
    assert pinyin('阿Q', style=TONE2) == [['a1'], ['Q']]
    assert pinyin('B超', style=TONE2) == [['B'], ['cha1o']]
    assert pinyin('AB超C', style=TONE2) == [['AB'], ['cha1o'], ['C']]
    assert pinyin('AB阿C', style=TONE2) == [['AB'], ['a1'], ['C']]
    assert pinyin('维生素C', style=TONE2) == \
        [['we2i'], ['she1ng'], ['su4'], ['C']]


def test_others():
    # 空字符串
    assert pinyin('') == []
    # 单个汉字
    assert pinyin('營') == [['y\xedng']]
    # 中国 人
    assert pinyin('中国人') == [['zh\u014dng'], ['gu\xf3'], ['r\xe9n']]
    # 日文
    assert pinyin('の') == [['\u306e']]
    # 没有读音的汉字，还不存在的汉字
    assert pinyin('\ufaff') == [['\ufaff']]


def test_lazy_pinyin():
    assert lazy_pinyin('中国人') == ['zhong', 'guo', 'ren']
    assert lazy_pinyin('中心') == ['zhong', 'xin']
    assert lazy_pinyin('中心', style=TONE) == ['zh\u014dng', 'x\u012bn']
    assert lazy_pinyin('中心', style=INITIALS) == ['zh', 'x']
    assert lazy_pinyin('中心', style=BOPOMOFO) == ['ㄓㄨㄥ', 'ㄒㄧㄣ']
    assert lazy_pinyin('中心', style=CYRILLIC) == ['чжун1', 'синь1']


def test_seg():
    hans = '音乐'
    hans_seg = list(seg(hans))
    assert pinyin(hans_seg, style=TONE2) == [['yi1n'], ['yue4']]
    # 中英文混合的固定词组
    assert pinyin('黄山B股', style=TONE2) == \
        [['hua2ng'], ['sha1n'], ['B'], ['gu3']]
    assert pinyin('A股', style=TONE2) == [['A'], ['gu3']]
    assert pinyin('阿Q', style=TONE2) == [['a1'], ['Q']]
    assert pinyin('B超', style=TONE2) == [['B'], ['cha1o']]
    assert pinyin('AB超C', style=TONE2) == [['AB'], ['cha1o'], ['C']]
    assert pinyin('AB阿C', style=TONE2) == [['AB'], ['a1'], ['C']]
    assert pinyin('维生素C', style=TONE2) == \
        [['we2i'], ['she1ng'], ['su4'], ['C']]


def test_custom_pinyin_dict():
    hans = '桔'
    try:
        assert lazy_pinyin(hans, style=TONE2) == ['ju2']
    except AssertionError:
        pass
    load_single_dict({ord('桔'): 'jú,jié'})
    assert lazy_pinyin(hans, style=TONE2) == ['ju2']


def test_custom_pinyin_dict2():
    hans = ['同行']
    try:
        assert lazy_pinyin(hans, style=TONE2) == ['to2ng', 'ha2ng']
    except AssertionError:
        pass
    load_phrases_dict({'同行': [['tóng'], ['xíng']]})
    assert lazy_pinyin(hans, style=TONE2) == ['to2ng', 'xi2ng']


def test_custom_pinyin_dict_tone2():
    load_single_dict({ord('桔'): 'ce4,si4'}, style='tone2')
    assert lazy_pinyin('桔', style=TONE2) == ['ce4']
    assert pinyin('桔') == [['cè']]


def test_custom_pinyin_dict2_tone2():
    load_phrases_dict({'同行': [['to4ng'], ['ku1']]}, style='tone2')
    assert lazy_pinyin(['同行'], style=TONE2) == ['to4ng', 'ku1']
    assert pinyin('同行') == [['tòng'], ['kū']]


def test_errors():
    hans = (
        ('啊', {'style': TONE2}, [['a']]),
        ('啊a', {'style': TONE2}, [['a'], ['a']]),
        # 非中文字符，没有拼音
        ('⺁', {'style': TONE2}, [['\u2e81']]),
        ('⺁', {'style': TONE2, 'errors': 'ignore'}, []),
        ('⺁', {'style': TONE2, 'errors': 'replace'}, [['2e81']]),
        ('⺁⺁', {'style': TONE2, 'errors': 'replace'}, [['2e812e81']]),
        ('⺁⺁', {'style': TONE2, 'errors': lambda x: ['a' for _ in x]},
         [['a'], ['a']]),
        ('⺁⺁', {'style': TONE2, 'errors': lambda x: [['a', 'b'], ['b', 'c']]},
         [['a'], ['b']]),
        ('⺁⺁', {'style': TONE2, 'heteronym': True,
                'errors': lambda x: [['a', 'b'], ['b', 'c']]},
         [['a', 'b'], ['b', 'c']]),
        # 中文字符，没有拼音
        ('鿅', {'style': TONE2}, [['\u9fc5']]),
        ('鿅', {'style': TONE2, 'errors': 'ignore'}, []),
        ('鿅', {'style': TONE2, 'errors': '233'}, []),
        ('鿅', {'style': TONE2, 'errors': 'replace'}, [['9fc5']]),
        ('鿅', {'style': TONE2, 'errors': lambda x: ['a']}, [['a']]),
        ('鿅', {'style': TONE2, 'errors': lambda x: None}, []),
        ('鿅鿅', {'style': TONE2, 'errors': lambda x: ['a' for _ in x]},
         [['a'], ['a']]),
        ('鿅鿅', {'style': TONE2, 'errors': lambda x: [['a', 'b']]},
         [['a'], ['a']]),
        ('鿅鿅', {'style': TONE2, 'heteronym': True,
                'errors': lambda x: [['a', 'b']]},
         [['a', 'b'], ['a', 'b']]),
    )
    for han in hans:
        assert pinyin(han[0], **han[1]) == han[2]


def test_errors_callable():
    def foobar(chars):
        return 'a' * len(chars)

    class Foobar(object):
        def __call__(self, chars):
            return 'a' * len(chars)

    n = 5
    assert pinyin('あ' * n, errors=foobar) == [['a' * n]]
    assert pinyin('あ' * n, errors=Foobar()) == [['a' * n]]


def test_errors_exception():
    with pytest.raises(PinyinNotFoundException):
        pinyin('あ', errors='exception')

    with pytest.raises(PinyinNotFoundException):
        lazy_pinyin('あ', errors='exception')


def test_simple_seg():
    data = {
        '北京abcc': 'be3i ji1ng abcc',
        '你好にほんごРусский язык': 'ni3 ha3o にほんごРусский язык',
    }
    for h, p in data.items():
        assert slug([h], style=TONE2, separator=' ') == p

    hans = '你好にほんごРусский язык'
    ret = 'ni3 ha3o'
    assert slug(hans, style=TONE2, separator=' ', errors=lambda x: None) == ret


data_for_update = [
    # 便宜的发音
    [
        ['便宜'], {'style': TONE2}, ['pia2n', 'yi']
    ],
    [
        ['便宜从事'], {'style': TONE2}, ['bia4n', 'yi2', 'co2ng', 'shi4']
    ],
    [
        ['便宜施行'], {'style': TONE2}, ['bia4n', 'yi2', 'shi1', 'xi2ng']
    ],
    [
        ['便宜货'], {'style': TONE2}, ['pia2n', 'yi2', 'huo4']
    ],
    [
        ['贪便宜'], {'style': TONE2}, ['ta1n', 'pia2n', 'yi2']
    ],
    [
        ['讨便宜'], {'style': TONE2}, ['ta3o', 'pia2n', 'yi2']
    ],
    [
        ['小便宜'], {'style': TONE2}, ['xia3o', 'pia2n', 'yi2']
    ],
    [
        ['占便宜'], {'style': TONE2}, ['zha4n', 'pia2n', 'yi2']
    ],
    #
    [
        '\u3400', {'style': TONE2}, ['qiu1'],  # CJK 扩展 A:[3400-4DBF]
    ],
    [
        '\u4E00', {'style': TONE2}, ['yi1'],   # CJK 基本:[4E00-9FFF]
    ],
    # [
    #     '\uFA29', {'style': TONE2}, ['da3o'],  # CJK 兼容:[F900-FAFF]
    # ],
    # 误把 yu 放到声母列表了
    ['鱼', {'style': TONE2}, ['yu2']],
    ['鱼', {'style': FINALS}, ['v']],
    ['鱼', {'style': BOPOMOFO}, ['ㄩˊ']],
    ['鱼', {'style': CYRILLIC}, ['юй']],
    ['雨', {'style': TONE2}, ['yu3']],
    ['雨', {'style': FINALS}, ['v']],
    ['雨', {'style': BOPOMOFO}, ['ㄩˇ']],
    ['雨', {'style': CYRILLIC}, ['юй']],
    ['元', {'style': TONE2}, ['yua2n']],
    ['元', {'style': FINALS}, ['van']],
    ['元', {'style': BOPOMOFO}, ['ㄩㄢˊ']],
    ['元', {'style': CYRILLIC}, ['юань2']],
    # y, w 也不是拼音, yu的韵母是v, yi的韵母是i, wu的韵母是u
    ['呀', {'style': INITIALS}, ['']],
    ['呀', {'style': TONE2}, ['ya']],
    ['呀', {'style': FINALS}, ['ia']],
    ['呀', {'style': BOPOMOFO}, ['ㄧㄚ˙']],
    ['呀', {'style': CYRILLIC}, ['я']],
    ['无', {'style': INITIALS}, ['']],
    ['无', {'style': TONE2}, ['wu2']],
    ['无', {'style': FINALS}, ['u']],
    ['无', {'style': FINALS_TONE}, ['ú']],
    ['无', {'style': BOPOMOFO}, ['ㄨˊ']],
    ['无', {'style': CYRILLIC}, ['у2']],
    ['衣', {'style': TONE2}, ['yi1']],
    ['衣', {'style': FINALS}, ['i']],
    ['衣', {'style': BOPOMOFO}, ['ㄧ']],
    ['衣', {'style': CYRILLIC}, ['и1']],
    ['万', {'style': TONE2}, ['wa4n']],
    ['万', {'style': FINALS}, ['uan']],
    ['万', {'style': BOPOMOFO}, ['ㄨㄢˋ']],
    ['万', {'style': CYRILLIC}, ['вань4']],
    # ju, qu, xu 的韵母应该是 v
    ['具', {'style': FINALS_TONE}, ['ǜ']],
    ['具', {'style': FINALS_TONE2}, ['v4']],
    ['具', {'style': FINALS}, ['v']],
    ['具', {'style': BOPOMOFO}, ['ㄐㄩˋ']],
    ['具', {'style': CYRILLIC}, ['цзюй4']],
    ['取', {'style': FINALS_TONE}, ['ǚ']],
    ['取', {'style': FINALS_TONE2}, ['v3']],
    ['取', {'style': FINALS}, ['v']],
    ['取', {'style': BOPOMOFO}, ['ㄑㄩˇ']],
    ['取', {'style': CYRILLIC}, ['цюй3']],
    ['徐', {'style': FINALS_TONE}, ['ǘ']],
    ['徐', {'style': FINALS_TONE2}, ['v2']],
    ['徐', {'style': FINALS}, ['v']],
    ['徐', {'style': BOPOMOFO}, ['ㄒㄩˊ']],
    ['徐', {'style': CYRILLIC}, ['сюй2']],
    # ń
    ['嗯', {'style': NORMAL}, ['n']],
    ['嗯', {'style': TONE}, ['ń']],
    ['嗯', {'style': TONE2}, ['n2']],
    ['嗯', {'style': INITIALS}, ['']],
    ['嗯', {'style': FIRST_LETTER}, ['n']],
    ['嗯', {'style': FINALS}, ['']],
    ['嗯', {'style': FINALS_TONE}, ['']],
    ['嗯', {'style': FINALS_TONE2}, ['']],
    ['嗯', {'style': BOPOMOFO}, ['ㄣˊ']],
    ['嗯', {'style': CYRILLIC}, ['н2']],
    # ḿ  \u1e3f  U+1E3F
    ['呣', {'style': NORMAL}, ['m']],
    ['呣', {'style': TONE}, ['ḿ']],
    ['呣', {'style': TONE2}, ['m2']],
    ['呣', {'style': INITIALS}, ['']],
    ['呣', {'style': FIRST_LETTER}, ['m']],
    ['呣', {'style': FINALS}, ['']],
    ['呣', {'style': FINALS_TONE}, ['']],
    ['呣', {'style': FINALS_TONE2}, ['']],
    ['呣', {'style': BOPOMOFO}, ['ㄇㄨˊ']],
    ['呣', {'style': CYRILLIC}, ['м2']],
    # 41
    ['彷徨', {}, ['pang', 'huang']],
    ['彷徨', {'style': CYRILLIC}, ['пан2', 'хуан2']],
    # 注音
    ['打量', {'style': BOPOMOFO}, ['ㄉㄚˇ', 'ㄌㄧㄤˋ']],
    ['黄山b股', {'style': BOPOMOFO}, ['ㄏㄨㄤˊ', 'ㄕㄢ', 'b', 'ㄍㄨˇ']],
    ['打量', {'style': CYRILLIC}, ['да3', 'лян4']],
    ['黄山b股', {'style': CYRILLIC}, ['хуан2', 'шань1', 'b', 'гу3']],
    # 50
    ['打量', {'style': TONE2}, ['da3', 'lia4ng']],
    ['打量', {'style': TONE3}, ['da3', 'liang4']],
    ['侵略', {'style': TONE2}, ['qi1n', 'lve4']],
    ['侵略', {'style': TONE3}, ['qin1', 'lve4']],
    ['侵略', {'style': FINALS_TONE2}, ['i1n', 've4']],
    ['侵略', {'style': FINALS_TONE3}, ['in1', 've4']],
    ['侵略', {'style': BOPOMOFO}, ['ㄑㄧㄣ', 'ㄌㄩㄝˋ']],
    ['侵略', {'style': CYRILLIC}, ['цинь1', 'люэ4']],
    ['〇', {'style': TONE}, ['líng']],
    # 二次分词
    [['你要', '重新考虑OK'], {'style': TONE}, [
        'nǐ', 'yào', 'chóng', 'xīn', 'kǎo', 'lǜ', 'OK']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_update)
def test_update(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result


@pytest.mark.skipif(not SUPPORT_UCS4, reason='dont support ucs4')
@pytest.mark.parametrize(
    'han, result', [
        ['\U00020000', ['he']],      # CJK 扩展 B:[20000-2A6DF]
        ['\U0002A79D', ['duo']],      # CJK 扩展 C:[2A700-2B73F]
        # ['\U0002B740', ['wu']],      # CJK 扩展 D:[2B740-2B81D]
        # ['\U0002F80A', ['seng']],    # CJK 兼容扩展:[2F800-2FA1F]
    ]
)
def test_support_ucs4(han, result):
    assert lazy_pinyin(han) == result


@pytest.mark.skipif(SUPPORT_UCS4, reason='support ucs4')
@pytest.mark.parametrize(
    'han', [
        '\U00020000',      # CJK 扩展 B:[20000-2A6DF]
        '\U0002A79D',      # CJK 扩展 C:[2A700-2B73F]
        # '\U0002B740',      # CJK 扩展 D:[2B740-2B81D]
        # '\U0002F80A',      # CJK 兼容扩展:[2F800-2FA1F]
    ]
)
def test_dont_support_ucs4(han):
    assert pinyin(han) == [[han]]


def test_36():
    hans = '两年前七斤喝醉了酒'
    pys = ['liang', 'nian', 'qian', 'qi', 'jin', 'he', 'zui', 'le', 'jiu']
    assert lazy_pinyin(hans) == pys


def test_with_unknown_style():
    assert lazy_pinyin('中国') == ['zhong', 'guo']
    assert lazy_pinyin('中国', style='unknown') == ['zhōng', 'guó']
    assert pinyin('中国') == [['zhōng'], ['guó']]
    assert pinyin('中国', style='unknown') == [['zhōng'], ['guó']]


@pytest.mark.parametrize('kwargs,result', [
    [{}, [['zh\u014dng', 'zh\xf2ng'], ['x\u012bn']]],
    [dict(strict=False), [['zh\u014dng', 'zh\xf2ng'], ['x\u012bn']]],
    [dict(style=NORMAL), [['zhong'], ['xin']]],
    [dict(style=NORMAL, strict=False), [['zhong'], ['xin']]],
    [dict(style=TONE), [['zh\u014dng', 'zh\xf2ng'], ['x\u012bn']]],
    [dict(style=TONE, strict=False), [
        ['zh\u014dng', 'zh\xf2ng'], ['x\u012bn']]],
    [dict(style=TONE2), [['zho1ng', 'zho4ng'], ['xi1n']]],
    [dict(style=TONE2, strict=False), [['zho1ng', 'zho4ng'], ['xi1n']]],
    [dict(style=TONE3), [['zhong1', 'zhong4'], ['xin1']]],
    [dict(style=TONE3, strict=False), [['zhong1', 'zhong4'], ['xin1']]],
    [dict(style=INITIALS), [['zh'], ['x']]],
    [dict(style=INITIALS, strict=False), [['zh'], ['x']]],
    [dict(style=FIRST_LETTER), [['z'], ['x']]],
    [dict(style=FIRST_LETTER, strict=False), [['z'], ['x']]],
    [dict(style=FINALS), [['ong'], ['in']]],
    [dict(style=FINALS, strict=False), [['ong'], ['in']]],
    [dict(style=FINALS_TONE), [['\u014dng', '\xf2ng'], ['\u012bn']]],
    [dict(style=FINALS_TONE, strict=False),  [
        ['\u014dng', '\xf2ng'], ['\u012bn']]],
    [dict(style=FINALS_TONE2), [['o1ng', 'o4ng'], ['i1n']]],
    [dict(style=FINALS_TONE2, strict=False),  [['o1ng', 'o4ng'], ['i1n']]],
    [dict(style=FINALS_TONE3), [['ong1', 'ong4'], ['in1']]],
    [dict(style=FINALS_TONE3, strict=False), [['ong1', 'ong4'], ['in1']]],
])
def test_heteronym_and_style(kwargs, result):
    hans = '中心'
    kwargs['heteronym'] = True
    assert pinyin(hans, **kwargs) == result


@pytest.mark.parametrize('kwargs,result', [
    [{}, [['zhāo'], ['yáng']]],
    [dict(heteronym=True), [['zhāo', 'cháo'], ['yáng']]],
    [dict(strict=False), [['zhāo'], ['yáng']]],
    [dict(strict=False, heteronym=True), [['zhāo', 'cháo'], ['yáng']]],
    [dict(style=NORMAL), [['zhao'], ['yang']]],
    [dict(style=NORMAL, heteronym=True), [['zhao', 'chao'], ['yang']]],
    [dict(style=NORMAL, strict=False), [['zhao'], ['yang']]],
    [dict(style=NORMAL, strict=False, heteronym=True), [['zhao', 'chao'],
                                                        ['yang']]],
    [dict(style=TONE), [['zhāo'], ['yáng']]],
    [dict(style=TONE, heteronym=True), [['zhāo', 'cháo'], ['yáng']]],
    [dict(style=TONE, strict=False), [['zhāo'], ['yáng']]],
    [dict(style=TONE, strict=False, heteronym=True), [['zhāo', 'cháo'],
                                                      ['yáng']]],
    [dict(style=TONE2), [['zha1o'], ['ya2ng']]],
    [dict(style=TONE2, heteronym=True), [['zha1o', 'cha2o'], ['ya2ng']]],
    [dict(style=TONE2, strict=False), [['zha1o'], ['ya2ng']]],
    [dict(style=TONE2, strict=False, heteronym=True), [['zha1o', 'cha2o'],
                                                       ['ya2ng']]],
    [dict(style=TONE3), [['zhao1'], ['yang2']]],
    [dict(style=TONE3, heteronym=True), [['zhao1', 'chao2'], ['yang2']]],
    [dict(style=TONE3, strict=False), [['zhao1'], ['yang2']]],
    [dict(style=TONE3, strict=False, heteronym=True), [['zhao1', 'chao2'],
                                                       ['yang2']]],
    [dict(style=INITIALS), [['zh'], ['']]],
    [dict(style=INITIALS, heteronym=True), [['zh', 'ch'], ['']]],
    [dict(style=INITIALS, strict=False), [['zh'], ['y']]],
    [dict(style=INITIALS, strict=False, heteronym=True), [['zh', 'ch'],
                                                          ['y']]],
    [dict(style=FIRST_LETTER), [['z'], ['y']]],
    [dict(style=FIRST_LETTER, heteronym=True), [['z', 'c'], ['y']]],
    [dict(style=FIRST_LETTER, strict=False), [['z'], ['y']]],
    [dict(style=FIRST_LETTER, strict=False, heteronym=True), [['z', 'c'],
                                                              ['y']]],
    [dict(style=FINALS), [['ao'], ['iang']]],
    [dict(style=FINALS, heteronym=True), [['ao'], ['iang']]],
    [dict(style=FINALS, strict=False), [['ao'], ['ang']]],
    [dict(style=FINALS, strict=False, heteronym=True), [['ao'], ['ang']]],
    [dict(style=FINALS_TONE), [['āo'], ['iáng']]],
    [dict(style=FINALS_TONE, heteronym=True), [['āo', 'áo'], ['iáng']]],
    [dict(style=FINALS_TONE, strict=False),  [['āo'], ['áng']]],
    [dict(style=FINALS_TONE, strict=False, heteronym=True),  [['āo', 'áo'],
                                                              ['áng']]],
    [dict(style=FINALS_TONE2), [['a1o'], ['ia2ng']]],
    [dict(style=FINALS_TONE2, heteronym=True), [['a1o', 'a2o'], ['ia2ng']]],
    [dict(style=FINALS_TONE2, strict=False),  [['a1o'], ['a2ng']]],
    [dict(style=FINALS_TONE2, strict=False, heteronym=True),  [['a1o', 'a2o'],
                                                               ['a2ng']]],
    [dict(style=FINALS_TONE3), [['ao1'], ['iang2']]],
    [dict(style=FINALS_TONE3, heteronym=True), [['ao1', 'ao2'], ['iang2']]],
    [dict(style=FINALS_TONE3, strict=False), [['ao1'], ['ang2']]],
    [dict(style=FINALS_TONE3, strict=False, heteronym=True), [['ao1', 'ao2'],
                                                              ['ang2']]],
])
def test_heteronym_and_style_phrase(kwargs, result):
    hans = '朝阳'
    assert pinyin(hans, **kwargs) == result


def test_m4():
    # U+5463: ḿ,móu,m̀  # 呣
    han = '呣'
    assert pinyin(han) == [['ḿ']]
    assert pinyin(han, heteronym=True) == [['ḿ', 'm̀', 'móu']]
    assert pinyin(
        han, heteronym=True, style=NORMAL) == [['m', 'mou']]
    assert pinyin(
        han, heteronym=True, style=TONE) == [['ḿ', 'm̀', 'móu']]
    assert pinyin(
        han, heteronym=True, style=TONE2) == [['m2', 'm4', 'mo2u']]
    assert pinyin(
        han, heteronym=True, style=TONE3) == [['m2', 'm4', 'mou2']]
    assert pinyin(
        han, heteronym=True, style=INITIALS) == [['m']]
    assert pinyin(
        han, heteronym=True, style=FIRST_LETTER) == [['m']]
    assert pinyin(
        han, heteronym=True, style=FINALS) == [['ou']]
    assert pinyin(
        han, heteronym=True, style=FINALS, strict=False) == [['m', 'ou']]
    assert pinyin(
        han, heteronym=True, style=FINALS_TONE) == [['óu']]
    assert pinyin(
        han, heteronym=True, style=FINALS_TONE, strict=False
    ) == [['ḿ', 'm̀', 'óu']]
    assert pinyin(
        han, heteronym=True, style=FINALS_TONE2) == [['o2u']]
    assert pinyin(
        han, heteronym=True, style=FINALS_TONE2, strict=False
    ) == [['m2', 'm4', 'o2u']]
    assert pinyin(
        han, heteronym=True, style=FINALS_TONE3) == [['ou2']]
    assert pinyin(
        han, heteronym=True, style=FINALS_TONE3, strict=False
    ) == [['m2', 'm4', 'ou2']]


@pytest.mark.parametrize('han,style,expect', [
    ['呣', Style.TONE, ['ḿ', 'm̀']],
    ['呣', Style.TONE2, ['m2', 'm4']],
    ['呣', Style.BOPOMOFO, ['ㄇㄨˊ', 'ㄇㄨˋ', 'ㄇㄡˊ']],

    ['嘸', Style.TONE, ['m̄', 'ḿ']],
    ['嘸', Style.TONE2, ['m1', 'm2']],
    ['嘸', Style.BOPOMOFO, ['ㄈㄨˇ', 'ㄨˇ', 'ㄇㄨ', 'ㄇㄨˊ']],

    ['誒', Style.TONE, ['ê̄', 'ế', 'ê̌', 'ề']],
    ['誒', Style.TONE2, ['ê1', 'ê2', 'ê3', 'ê4']],
    ['誒', Style.BOPOMOFO, ['ㄟˊ', 'ㄒㄧ', 'ㄧˋ', 'ㄝ', 'ㄝˊ',
                           'ㄝˇ', 'ㄟˇ', 'ㄝˋ', 'ㄟˋ', 'ㄟ']],
])
def test_m_e(han, style, expect):
    result = pinyin(han, style=style, heteronym=True)
    assert len(result) == 1
    assert (set(result[0]) & set(expect)) == set(expect)


@pytest.mark.parametrize('han,expect', [
    ['北京', ['pei', 'ching']],
    ['台北', ["t'ai", 'pei']],
    ['太极', ["t'ai", 'chi']],
    ['只', ['chih']],
    ['韦氏拼音', ['wei', 'shih', "p'in", 'yin']],
    ['威妥玛拼音', ['wei', "t'o", 'ma', "p'in", 'yin']],
])
def test_wadegiles(han, expect):
    got = lazy_pinyin(han, style=Style.WADEGILES)
    assert got == expect


@pytest.mark.parametrize('han,v_to_u,expect', [
    ['句', False, ['chv']],
    ['句', True, ['chü']],
])
def test_wadegiles_v_u(han, v_to_u, expect):
    got = lazy_pinyin(han, v_to_u=v_to_u, style=Style.WADEGILES)
    assert got == expect


@pytest.mark.parametrize('han,expect', [
    ['国语罗马字', [['gwo'], ['yeu'], ['luo'], ['maa'], ['tzyh']]],
])
def test_gwoyeu(han, expect):
    got = pinyin(han, style=Style.GWOYEU)
    assert got == expect


@pytest.mark.parametrize('han,style,expect', [
    ['中国', Style.BRAILLE_MAINLAND,
     [['⠌⠲'], ['⠛⠕']]],
    ['中国', Style.BRAILLE_MAINLAND_TONE,
     [['⠌⠲⠁'], ['⠛⠕⠂']]],
    ['时间不早了', Style.BRAILLE_MAINLAND,
     [['⠱'], ['⠛⠩'], ['⠃⠥'], ['⠵⠖'], ['⠇⠢']]],
    ['时间不早了', Style.BRAILLE_MAINLAND_TONE,
     [['⠱⠂'], ['⠛⠩⠁'], ['⠃⠥⠆'], ['⠵⠖⠄'], ['⠇⠢']]],
])
def test_braille(han, style, expect):
    got = pinyin(han, style=style)
    assert got == expect


if __name__ == '__main__':
    import pytest
    pytest.cmdline.main()



from typing import Any, Text, Tuple

GWOYEU_REPLACE = ... # type: Tuple[Tuple[Any]]

class GwoyeuConverter(object):
    def to_wade_glides(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def _pre_convert(self, pinyin: Text) -> Text: ...

converter = ... # type: GwoyeuConverter



from typing import Union, Text, ByteString


class PinyinNotFoundException(Exception):
    message: Union[Text, ByteString]
    chars: Union[Text, ByteString]

    def __init__(self, chars: Union[Text, ByteString]) -> None: ...



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from copy import deepcopy

from pypinyin import pinyin, Style
from pypinyin.style import register, convert


def test_custom_style_with_decorator():
    style_value = 'test_custom_style_with_decorator'

    @register(style_value)
    def func(pinyin, **kwargs):
        return pinyin + str(len(pinyin))

    hans = '北京'
    origin_pinyin_s = pinyin(hans)
    expected_pinyin_s = deepcopy(origin_pinyin_s)
    for pinyin_s in expected_pinyin_s:
        for index, py in enumerate(pinyin_s):
            pinyin_s[index] = func(py)

    assert pinyin(hans, style=style_value) == expected_pinyin_s


def test_custom_style_with_call():
    style_value = 'test_custom_style_with_call'

    def func(pinyin, **kwargs):
        return str(len(pinyin))

    register(style_value, func=func)

    hans = '北京'
    origin_pinyin_s = pinyin(hans)
    expected_pinyin_s = deepcopy(origin_pinyin_s)
    for pinyin_s in expected_pinyin_s:
        for index, py in enumerate(pinyin_s):
            pinyin_s[index] = func(py)

    assert pinyin(hans, style=style_value) == expected_pinyin_s


def test_custom_style_with_han():
    style_value = 'test_custom_style_with_han'

    @register(style_value)
    def func(pinyin, **kwargs):
        return kwargs.get('han', '') + pinyin

    hans = '北京'
    expected_pinyin_s = [['北běi'], ['京jīng']]

    assert pinyin(hans, style=style_value) == expected_pinyin_s


def test_finals_tone3_no_final():
    assert convert('ń', Style.FINALS_TONE3, True, None) == ''
    assert convert('ń', Style.FINALS_TONE3, False, None) == 'n2'


def test_issue_291():
    assert pinyin('誒', style=Style.TONE, heteronym=True) == \
           [['éi', 'xī', 'yì', 'ê̄', 'ế', 'ê̌', 'ěi',
             'ề', 'èi', 'ēi']]
    assert pinyin('誒', style=Style.BOPOMOFO, heteronym=True) == \
           [['ㄟˊ', 'ㄒㄧ', 'ㄧˋ', 'ㄝ', 'ㄝˊ', 'ㄝˇ', 'ㄟˇ',
             'ㄝˋ', 'ㄟˋ', 'ㄟ']]

    assert pinyin('欸', style=Style.TONE, heteronym=True) == \
           [['āi', 'ǎi', 'ê̄', 'ế', 'ê̌', 'ề', 'xiè', 'éi',
             'ěi', 'èi', 'ēi']]
    assert pinyin('欸', style=Style.BOPOMOFO, heteronym=True) == \
           [['ㄞ', 'ㄞˇ', 'ㄝ', 'ㄝˊ', 'ㄝˇ', 'ㄝˋ', 'ㄒㄧㄝˋ', 'ㄟˊ',
             'ㄟˇ', 'ㄟˋ', 'ㄟ']]


if __name__ == '__main__':
    import pytest
    pytest.cmdline.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re

from pypinyin.constants import Style
from pypinyin.style import register
from pypinyin.style._constants import RE_TONE3
from pypinyin.style._utils import replace_symbol_to_number

GWOYEU_REPLACE = (
    (re.compile(r'^r5$'), 'er5'),
    (re.compile(r'iu'), 'iou'),
    (re.compile(r'ao'), 'au'),
    (re.compile(r'^yi?'), 'i'),
    (re.compile(r'^wu?'), 'u'),
    (re.compile(r'^([jqx])u'), '\\1iu'),
    (re.compile(r'(?<![iy])u([in])'), 'ue\\1'),
    (re.compile(r'v'), 'iu'),
    (re.compile(r'^([zcsr]h?)i'), '\\1y'),
    (re.compile(r'^zh'), 'j'),
    (re.compile(r'^z'), 'tz'),
    (re.compile(r'^c(?!h)'), 'ts'),
    (re.compile(r'^q'), 'ch'),
    (re.compile(r'^x'), 'sh'),
    (re.compile(r'er'), 'el'),
    (re.compile(r'5$'), ''),
    (re.compile(r'0$'), 'q'),
    (re.compile(r'^i(.*[34])$'), 'yi\\1'),
    (re.compile(r'^u(.*[34])$'), 'wu\\1'),
    (re.compile(r'^yi([aeu].*4)$'), 'y\\1'),
    (re.compile(r'^wu([ae].*4)$'), 'w\\1'),
)

TONE_REPLACE = (
    (re.compile(r'^([lmnr])(.+)1$'), '\\1h\\2'),
    (re.compile(r'1$'), ''),
    (re.compile(r'^([lmnr])(.+)2$'), '\\1\\2'),
    (re.compile(r'^([^ae]*)i(ng?)*2$'), '\\1yi\\2'),
    (re.compile(r'^([^ao]*)u2$'), '\\1wu'),
    (re.compile(r'^([^ae]*)i(.+)2$'), '\\1y\\2'),
    (re.compile(r'^([^ao]*)u(.+)2$'), '\\1w\\2'),
    (re.compile(r'([aeiouy]+)(.*)2$'), '\\1r\\2'),
    (re.compile(r'^([^aeiou]*)([iu])(ng?)?3$'), '\\1\\2\\2\\3'),
    (re.compile(r'^([^eu]*)i(.*)3$'), '\\1e\\2'),
    (re.compile(r'^(.*)u(.*)3$'), '\\1o\\2'),
    (re.compile(r'([aeiouy])(.*)3$'), '\\1\\1\\2'),
    (re.compile(r'^([^ae]*)i4$'), '\\1ih'),
    (re.compile(r'^([^ao]*)u4$'), '\\1uh'),
    (re.compile(r'i4$'), 'y'),
    (re.compile(r'u4$'), 'w'),
    (re.compile(r'l4$'), 'll'),
    (re.compile(r'ng4$'), 'nq'),
    (re.compile(r'n4$'), 'nn'),
    (re.compile(r'4$'), 'h'),
)


class GwoyeuConverter(object):
    def _pre_convert(self, pinyin):
        # 用数字表示声调
        pinyin = replace_symbol_to_number(pinyin)
        # 将声调数字移动到最后
        return RE_TONE3.sub(r'\1\3\2', pinyin)

    def to_gwoyeu(self, pinyin, **kwargs):
        pinyin = self._pre_convert(pinyin)
        for find_re, replace in GWOYEU_REPLACE:
            pinyin = find_re.sub(replace, pinyin)
        for find_re, replace in TONE_REPLACE:
            if find_re.search(pinyin):
                return find_re.sub(replace, pinyin)
        return pinyin


converter = GwoyeuConverter()

register(Style.GWOYEU, func=converter.to_gwoyeu)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pypinyin.constants import Style
from pypinyin.converter import DefaultConverter


def test_pre_convert_style_return_value():
    class A(DefaultConverter):
        def pre_convert_style(self, han, orig_pinyin, style, strict, **kwargs):
            return 'test'

    han = '测试'
    assert DefaultConverter().convert(
        han, Style.TONE2, False, 'ignore', True) == [['ce4'], ['shi4']]
    assert A().convert(
        han, Style.TONE2, False, 'ignore', True) == [['test'], ['test']]


def test_post_convert_style_return_value():
    class A(DefaultConverter):
        def post_convert_style(self, han, orig_pinyin, converted_pinyin,
                               style, strict, **kwargs):
            return 'test'

    han = '测试'
    assert DefaultConverter().convert(
        han, Style.TONE2, False, 'ignore', True) == [['ce4'], ['shi4']]
    assert A().convert(
        han, Style.TONE2, False, 'ignore', True) == [['test'], ['test']]


def test_pre_handle_nopinyin_return_value():
    class A(DefaultConverter):
        def pre_handle_nopinyin(self, chars, style, heteronym, errors,
                                strict, **kwargs):
            return 'abc'

    han = 'test'
    assert DefaultConverter().convert(
        han, Style.TONE2, False, 'default', True) == [['test']]
    assert A().convert(
        han, Style.TONE2, False, 'default', True) == [['abc']]


def test_post_handle_nopinyin_return_value():
    class A(DefaultConverter):
        def post_handle_nopinyin(self, chars, style, heteronym, errors,
                                 strict, pinyin, **kwargs):
            return 'abc'

    han = 'test'
    assert DefaultConverter().convert(
        han, Style.TONE2, False, 'default', True) == [['test']]
    assert A().convert(
        han, Style.TONE2, False, 'default', True) == [['abc']]


def test_post_pinyin_return_value_single_pinyin():
    class A(DefaultConverter):
        def post_pinyin(self, han, heteronym, pinyin, **kwargs):
            return {
                '测': [['zhāo']],
                '试': [['yáng']],
                '测试': [['zhāo'], ['yáng']],
            }[han]

    han = '测试'
    assert DefaultConverter().convert(
        han, Style.TONE3, False, 'ignore', True) == [['ce4'], ['shi4']]
    assert A().convert(
        han, Style.TONE3, False, 'ignore', True) == [['zhao1'], ['yang2']]


def test_post_pinyin_return_value_phrase_pinyin():
    class A(DefaultConverter):
        def post_pinyin(self, han, heteronym, pinyin, **kwargs):
            return {
                '北': [['zhāo']],
                '京': [['yáng']],
                '北京': [['zhāo'], ['yáng']],
            }[han]

    han = '北京'
    assert DefaultConverter().convert(
        han, Style.TONE3, False, 'ignore', True) == [['bei3'], ['jing1']]
    assert A().convert(
        han, Style.TONE3, False, 'ignore', True) == [['zhao1'], ['yang2']]



# reference: https://github.com/pyinstaller/hooksample/blob/master/src/pyi_hooksample/__pyinstaller/hook-pyi_hooksample.py  # noqa

from PyInstaller.utils.hooks import collect_data_files  # type: ignore

datas = collect_data_files(
        'pypinyin', excludes=['__pyinstaller'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pypinyin.constants import Style
from pypinyin.core import (
    Pinyin, to_fixed, handle_nopinyin, single_pinyin, phrase_pinyin)


def test_use_pre_seg_to_skip_seg():
    class A(Pinyin):
        def pre_seg(self, hans, **kwargs):
            return ['a', 'b', 'c']

    mypinyin = A()

    assert Pinyin().pinyin('测试') == [['cè'], ['shì']]
    assert mypinyin.pinyin('测试') == [['a'], ['b'], ['c']]


def test_use_post_seg_to_change_seg_result():
    class A(Pinyin):
        def post_seg(self, hans, seg_data, **kwargs):
            return ['a', 'b', 'c']

    mypinyin = A()

    assert Pinyin().pinyin('测试') == [['cè'], ['shì']]
    assert mypinyin.pinyin('测试') == [['a'], ['b'], ['c']]


def test_use_seg_function_change_seg_func():
    def seg(han):
        return ['a', 'b', 'c']

    class A(Pinyin):
        def get_seg(self):
            return seg

    mypinyin = A()

    assert Pinyin().pinyin('测试') == [['cè'], ['shì']]
    assert mypinyin.pinyin('测试') == [['a'], ['b'], ['c']]


def test_to_fixed_for_compatibly():
    assert to_fixed('cè', Style.INITIALS) == 'c'


def test_handle_nopinyin_for_compatibly():
    assert handle_nopinyin('test') == [['test']]


def test_single_pinyin_for_compatibly():
    assert single_pinyin('测', Style.TONE, False) == [['cè']]


def test_phrase_pinyin_for_compatibly():
    assert phrase_pinyin('测试', Style.TONE, False) == [['cè'], ['shì']]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import copy
try:
    from importlib import reload
except ImportError:
    pass
import os
import sys

import pytest


@pytest.fixture(scope='function')
def cleanup():
    _clean()
    try:
        yield
    finally:
        _clean()


def _clean():
    for module in copy.copy(sys.modules):
        if module.startswith('pypinyin'):
            sys.modules.pop(module, None)


def test_env(cleanup):
    os.environ['PYPINYIN_NO_PHRASES'] = 'true'
    import pypinyin.core  # noqa

    assert pypinyin.core.PHRASES_DICT == {}
    assert pypinyin.core.seg('北京') == ['北京']


def test_no_copy(cleanup):
    """ 禁用copy操作的测试 """
    import pypinyin.core  # noqa

    assert pypinyin.core.PINYIN_DICT is not pypinyin.pinyin_dict.pinyin_dict

    os.environ['PYPINYIN_NO_DICT_COPY'] = 'true'
    reload(pypinyin.constants)
    assert pypinyin.constants.PINYIN_DICT is pypinyin.pinyin_dict.pinyin_dict



# -*- coding: utf-8 -*-
"""中国内地盲文相关的几个拼音风格实现:

Style.BRAILLE_MAINLAND
Style.BRAILLE_MAINLAND_TONE
"""
from typing import Any, Dict, Tuple, Text

BRAILLE_MAINLAND_REPLACE = ...  # type: Tuple[Tuple[Any]]

BRAILLE_MAINLAND_TABLE = ...  # type: Dict[Text, Text]

class BrailleMainlandConverter(object):
    def to_braille_mainland_tone(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_braille_mainland(self, pinyin: Text, **kwargs: Any) -> Text: ...

converter = ...  # type: BrailleMainlandConverter



# -*- coding: utf-8 -*-
"""中国内地盲文相关的几个拼音风格实现:

Style.BRAILLE_MAINLAND
Style.BRAILLE_MAINLAND_TONE

https://en.wikipedia.org/wiki/Mainland_Chinese_Braille
"""
from __future__ import unicode_literals
import re

from pypinyin.constants import Style
from pypinyin.style import register
from pypinyin.style._constants import RE_TONE3
from pypinyin.style._utils import (
    replace_symbol_to_number, replace_symbol_to_no_symbol)

BRAILLE_MAINLAND_REPLACE = (
    (re.compile(r'iu'), 'iou'),
    (re.compile(r'ui'), 'uei'),
    (re.compile(r'un'), 'uen'),
    (re.compile(r'ong'), 'ung'),
    (re.compile(r'^yi?'), 'i'),
    (re.compile(r'^wu?'), 'u'),
    (re.compile(r'y'), 'i'),
    (re.compile(r'w'), 'u'),
    (re.compile(r'iu'), 'v'),
    (re.compile(r'^([jqx])u'), '\\1v'),
    (re.compile(r'([iuv])n'), '\\1en'),
    (re.compile(r'^zhi?'), 'Z'),
    (re.compile(r'^chi?'), 'C'),
    (re.compile(r'^shi?'), 'S'),
    (re.compile(r'^([zcsr])i'), '\\1'),
    (re.compile(r'iang'), '⠭'),
    (re.compile(r'uang'), '⠶'),
    (re.compile(r'ueng'), '⠲'),
    (re.compile(r'iong'), '⠹'),
    (re.compile(r'ang'), '⠦'),
    (re.compile(r'eng'), '⠼'),
    (re.compile(r'uai'), '⠽'),
    (re.compile(r'iao'), '⠜'),
    (re.compile(r'iou'), '⠳'),
    (re.compile(r'ian'), '⠩'),
    (re.compile(r'uan'), '⠻'),
    (re.compile(r'van'), '⠯'),
    (re.compile(r'uen'), '⠒'),
    (re.compile(r'ing'), '⠡'),
    (re.compile(r'ong'), '⠲'),
    (re.compile(r'er'), '⠗'),
    (re.compile(r'ai'), '⠪'),
    (re.compile(r'ei'), '⠮'),
    (re.compile(r'ao'), '⠖'),
    (re.compile(r'ou'), '⠷'),
    (re.compile(r'an'), '⠧'),
    (re.compile(r'en'), '⠴'),
    (re.compile(r'ia'), '⠫'),
    (re.compile(r'ua'), '⠿'),
    (re.compile(r'ie'), '⠑'),
    (re.compile(r'uo'), '⠕'),
    (re.compile(r've'), '⠾'),
    (re.compile(r'ui'), '⠺'),
    (re.compile(r'in'), '⠣'),
    (re.compile(r'vn'), '⠸'),
)

BRAILLE_MAINLAND_TABLE = dict(zip(
    'bpmfdtnlgkhjqxZCSrzcsiuvaoe1234',
    '⠃⠏⠍⠋⠙⠞⠝⠇⠛⠅⠓⠛⠅⠓⠌⠟⠱⠚⠵⠉⠎⠊⠥⠬⠔⠢⠢⠁⠂⠄⠆'
))


class BrailleMainlandConverter(object):
    def to_braille_mainland_tone(self, pinyin, **kwargs):
        # 用数字表示声调
        pinyin = replace_symbol_to_number(pinyin)
        # 将声调数字移动到最后
        pinyin = RE_TONE3.sub(r'\1\3\2', pinyin)
        for find_re, replace in BRAILLE_MAINLAND_REPLACE:
            pinyin = find_re.sub(replace, pinyin)
        pinyin = ''.join(BRAILLE_MAINLAND_TABLE.get(x, x) for x in pinyin)
        return pinyin

    def to_braille_mainland(self, pinyin, **kwargs):
        pinyin = replace_symbol_to_no_symbol(pinyin)
        for find_re, replace in BRAILLE_MAINLAND_REPLACE:
            pinyin = find_re.sub(replace, pinyin)
        pinyin = ''.join(BRAILLE_MAINLAND_TABLE.get(x, x) for x in pinyin)
        return pinyin


converter = BrailleMainlandConverter()
register(Style.BRAILLE_MAINLAND_TONE, func=converter.to_braille_mainland_tone)
register(Style.BRAILLE_MAINLAND, func=converter.to_braille_mainland)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import sys
from pypinyin.compat import SUPPORT_UCS4
from pypinyin import runner
from pypinyin.tools import toneconvert
from pytest import mark


class Buffer(object):
    def __init__(self):
        self._data = []

    def write(self, data):
        self._data.append(data)


def test_runner_default():
    options = runner.get_parser().parse_args(['你好'])
    assert options.func == 'pinyin'
    assert options.style == 'zh4ao'
    assert options.separator == '-'
    assert not options.heteronym
    assert options.hans == ['你好']
    assert options.errors == 'default'


def test_runner_custom():
    options = runner.get_parser().parse_args([
        '--func', 'slug', '--style', 'zhao', '--separator', ' ',
        '--errors', 'ignore', '--heteronym', '你好啊'])
    assert options.func == 'slug'
    assert options.style == 'zhao'
    assert options.separator == ' '
    assert options.errors == 'ignore'
    assert options.heteronym
    assert options.hans == ['你好啊']


@mark.parametrize('args,output', [
    [['to-normal', 'yí,yì'], ['yi,yi', '\n']],
    [['to-tone', 'yi2,yi4'], ['yí,yì', '\n']],
    [['to-tone', 'hao3'], ['hǎo', '\n']],
    [['to-tone', 'zhong4 xin1'], ['zhòng xīn', '\n']],
    [['to-tone2', 'hǎo'], ['ha3o', '\n']],
    [['to-tone3', 'hǎo'], ['hao3', '\n']],
])
def test_toneconvert_default(args, output):
    buf = Buffer()
    sys.stdout = sys.stderr = buf
    toneconvert.main(args)
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__

    assert buf._data == output


@mark.skipif(not SUPPORT_UCS4, reason='dont support ucs4')
@mark.parametrize('args,output', [
    [['to-normal', 'm̄, ḿ, m̀, ê̄, ế, ê̌'], ['m, m, m, ê, ê, ê', '\n']],
    [['to-tone', 'm1, m2, m4, ê1, ê2, ê3'], ['m̄, ḿ, m̀, ê̄, ế, ê̌', '\n']],
    [['to-tone2', 'm̄, ḿ, m̀, ê̄, ế, ê̌'], ['m1, m2, m4, ê1, ê2, ê3', '\n']],
    [['to-tone3', 'm̄, ḿ, m̀, ê̄, ế, ê̌'], ['m1, m2, m4, ê1, ê2, ê3', '\n']],
])
def test_toneconvert_nme(args, output):
    buf = Buffer()
    sys.stdout = sys.stderr = buf
    toneconvert.main(args)
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__

    assert buf._data == output


if __name__ == '__main__':
    import pytest
    pytest.cmdline.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from itertools import chain

import pytest

from pypinyin import (
    lazy_pinyin, pinyin, NORMAL, TONE, TONE2, TONE3, INITIALS,
    FIRST_LETTER, FINALS, FINALS_TONE, FINALS_TONE2, FINALS_TONE3,
    pinyin_dict, Style
)
from pypinyin.compat import SUPPORT_UCS4, PY2
from pypinyin.style._constants import _FINALS
from pypinyin.contrib.tone_convert import to_initials, to_finals

# test data from http://www.moe.edu.cn/s78/A19/yxs_left/moe_810/s230/195802/t19580201_186000.html  # noqa
# 声母表
data_for_initials = [
    ['玻', dict(style=INITIALS), ['b']],
    ['坡', dict(style=INITIALS), ['p']],
    ['摸', dict(style=INITIALS), ['m']],
    ['佛', dict(style=INITIALS), ['f']],
    ['得', dict(style=INITIALS), ['d']],
    ['特', dict(style=INITIALS), ['t']],
    ['讷', dict(style=INITIALS), ['n']],
    ['勒', dict(style=INITIALS), ['l']],
    ['哥', dict(style=INITIALS), ['g']],
    ['科', dict(style=INITIALS), ['k']],
    ['喝', dict(style=INITIALS), ['h']],
    ['基', dict(style=INITIALS), ['j']],
    ['欺', dict(style=INITIALS), ['q']],
    ['希', dict(style=INITIALS), ['x']],
    ['知', dict(style=INITIALS), ['zh']],
    ['蚩', dict(style=INITIALS), ['ch']],
    ['诗', dict(style=INITIALS), ['sh']],
    ['日', dict(style=INITIALS), ['r']],
    ['资', dict(style=INITIALS), ['z']],
    ['雌', dict(style=INITIALS), ['c']],
    ['思', dict(style=INITIALS), ['s']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_initials)
def test_initials(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result
    assert list(chain(*pinyin(hans, **kwargs))) == result
    assert to_initials(lazy_pinyin(hans)[0]) == result[0]


# 韵母表
data_for_finals = [
    ['衣', dict(style=FINALS), ['i']],
    ['乌', dict(style=FINALS), ['u']],
    ['迂', dict(style=FINALS), ['v']],
    ['啊', dict(style=FINALS), ['a']],
    ['呀', dict(style=FINALS), ['ia']],
    ['蛙', dict(style=FINALS), ['ua']],
    ['喔', dict(style=FINALS), ['o']],
    ['窝', dict(style=FINALS), ['uo']],
    ['鹅', dict(style=FINALS), ['e']],
    ['耶', dict(style=FINALS), ['ie']],
    ['约', dict(style=FINALS), ['ve']],
    ['哀', dict(style=FINALS), ['ai']],
    ['歪', dict(style=FINALS), ['uai']],
    # ['欸', dict(style=FINALS), ['ei']],
    ['诶', dict(style=FINALS), ['ei']],
    ['威', dict(style=FINALS), ['uei']],
    ['熬', dict(style=FINALS), ['ao']],
    ['腰', dict(style=FINALS), ['iao']],
    ['欧', dict(style=FINALS), ['ou']],
    ['忧', dict(style=FINALS), ['iou']],
    ['安', dict(style=FINALS), ['an']],
    ['烟', dict(style=FINALS), ['ian']],
    ['弯', dict(style=FINALS), ['uan']],
    ['冤', dict(style=FINALS), ['van']],
    ['恩', dict(style=FINALS), ['en']],
    ['因', dict(style=FINALS), ['in']],
    ['温', dict(style=FINALS), ['uen']],
    ['晕', dict(style=FINALS), ['vn']],
    ['昂', dict(style=FINALS), ['ang']],
    ['央', dict(style=FINALS), ['iang']],
    ['汪', dict(style=FINALS), ['uang']],
    ['亨', dict(style=FINALS), ['eng']],
    ['英', dict(style=FINALS), ['ing']],
    ['翁', dict(style=FINALS), ['ueng']],
    ['轰', dict(style=FINALS), ['ong']],
    ['雍', dict(style=FINALS), ['iong']],
    ['儿', dict(style=FINALS), ['er']],
    ['欸', dict(style=FINALS, heteronym=True), ['ai', 'ê', 'ie', 'ei']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_finals)
def test_finals(hans, kwargs, result):
    if not kwargs.get('heteronym'):
        assert lazy_pinyin(hans, **kwargs) == result
        assert to_finals(lazy_pinyin(hans)[0]) == result[0]
    assert pinyin(hans, **kwargs) == [result]


# 零声母
data_for_zero_consonant = [
    # i行的韵母，前面没有声母的时候，写成yi(衣)，ya(呀)，ye(耶)，yao(腰)，
    # you(忧)，yan(烟)，yin(因)，yang(央)，ying(英)，yong(雍)。

    ['衣', dict(style=NORMAL), ['yi']],
    ['衣', dict(style=FINALS), ['i']],
    ['衣', dict(style=FINALS, strict=False), ['i']],
    ['呀', dict(style=NORMAL), ['ya']],
    ['呀', dict(style=FINALS), ['ia']],
    ['呀', dict(style=FINALS, strict=False), ['a']],
    ['耶', dict(style=NORMAL), ['ye']],
    ['耶', dict(style=FINALS), ['ie']],
    ['耶', dict(style=FINALS, strict=False), ['e']],
    ['腰', dict(style=NORMAL), ['yao']],
    ['腰', dict(style=FINALS), ['iao']],
    ['腰', dict(style=FINALS, strict=False), ['ao']],
    ['忧', dict(style=NORMAL), ['you']],
    ['忧', dict(style=FINALS), ['iou']],
    ['忧', dict(style=FINALS, strict=False), ['ou']],
    ['烟', dict(style=NORMAL), ['yan']],
    ['烟', dict(style=FINALS), ['ian']],
    ['烟', dict(style=FINALS, strict=False), ['an']],
    ['因', dict(style=NORMAL), ['yin']],
    ['因', dict(style=FINALS), ['in']],
    ['因', dict(style=FINALS, strict=False), ['in']],
    ['央', dict(style=NORMAL), ['yang']],
    ['央', dict(style=FINALS), ['iang']],
    ['央', dict(style=FINALS, strict=False), ['ang']],
    ['英', dict(style=NORMAL), ['ying']],
    ['英', dict(style=FINALS), ['ing']],
    ['英', dict(style=FINALS, strict=False), ['ing']],
    ['雍', dict(style=NORMAL), ['yong']],
    ['雍', dict(style=FINALS), ['iong']],
    ['雍', dict(style=FINALS, strict=False), ['ong']],

    ['宜', dict(style=NORMAL), ['yi']],
    ['宜', dict(style=NORMAL, strict=False), ['yi']],
    ['宜', dict(style=TONE), ['yí']],
    ['宜', dict(style=TONE, strict=False), ['yí']],
    ['宜', dict(style=TONE2), ['yi2']],
    ['宜', dict(style=TONE2, strict=False), ['yi2']],
    ['宜', dict(style=TONE3), ['yi2']],
    ['宜', dict(style=TONE3, strict=False), ['yi2']],
    ['宜', dict(style=INITIALS), ['']],
    ['宜', dict(style=INITIALS, strict=False), ['y']],
    ['宜', dict(style=FIRST_LETTER), ['y']],
    ['宜', dict(style=FIRST_LETTER, strict=False), ['y']],
    ['宜', dict(style=FINALS), ['i']],
    ['宜', dict(style=FINALS, strict=False), ['i']],
    ['宜', dict(style=FINALS_TONE), ['í']],
    ['宜', dict(style=FINALS_TONE, strict=False), ['í']],
    ['宜', dict(style=FINALS_TONE2), ['i2']],
    ['宜', dict(style=FINALS_TONE2, strict=False), ['i2']],
    ['宜', dict(style=FINALS_TONE3), ['i2']],
    ['宜', dict(style=FINALS_TONE3, strict=False), ['i2']],

    ['盐', dict(style=NORMAL), ['yan']],
    ['盐', dict(style=NORMAL, strict=False), ['yan']],
    ['盐', dict(style=TONE), ['yán']],
    ['盐', dict(style=TONE, strict=False), ['yán']],
    ['盐', dict(style=TONE2), ['ya2n']],
    ['盐', dict(style=TONE2, strict=False), ['ya2n']],
    ['盐', dict(style=TONE3), ['yan2']],
    ['盐', dict(style=TONE3, strict=False), ['yan2']],
    ['盐', dict(style=INITIALS), ['']],
    ['盐', dict(style=INITIALS, strict=False), ['y']],
    ['盐', dict(style=FIRST_LETTER), ['y']],
    ['盐', dict(style=FIRST_LETTER, strict=False), ['y']],
    ['盐', dict(style=FINALS), ['ian']],
    ['盐', dict(style=FINALS, strict=False), ['an']],
    ['盐', dict(style=FINALS_TONE), ['ián']],
    ['盐', dict(style=FINALS_TONE, strict=False), ['án']],
    ['盐', dict(style=FINALS_TONE2), ['ia2n']],
    ['盐', dict(style=FINALS_TONE2, strict=False), ['a2n']],
    ['盐', dict(style=FINALS_TONE3), ['ian2']],
    ['盐', dict(style=FINALS_TONE3, strict=False), ['an2']],


    # u行的韵母，前面没有声母的时候，写成wu(乌)，wa(蛙)，wo(窝)，wai(歪)，
    # wei(威)，wan(弯)，wen(温)，wang(汪)，weng(翁)。
    ['乌', dict(style=NORMAL), ['wu']],
    ['乌', dict(style=FINALS), ['u']],
    ['乌', dict(style=FINALS, strict=False), ['u']],
    ['蛙', dict(style=NORMAL), ['wa']],
    ['蛙', dict(style=FINALS), ['ua']],
    ['蛙', dict(style=FINALS, strict=False), ['a']],
    ['窝', dict(style=NORMAL), ['wo']],
    ['窝', dict(style=FINALS), ['uo']],
    ['窝', dict(style=FINALS, strict=False), ['o']],
    ['歪', dict(style=NORMAL), ['wai']],
    ['歪', dict(style=FINALS), ['uai']],
    ['歪', dict(style=FINALS, strict=False), ['ai']],
    ['威', dict(style=NORMAL), ['wei']],
    ['威', dict(style=FINALS), ['uei']],
    ['威', dict(style=FINALS, strict=False), ['ei']],
    ['弯', dict(style=NORMAL), ['wan']],
    ['弯', dict(style=FINALS), ['uan']],
    ['弯', dict(style=FINALS, strict=False), ['an']],
    ['温', dict(style=NORMAL), ['wen']],
    ['温', dict(style=FINALS), ['uen']],
    ['温', dict(style=FINALS, strict=False), ['en']],
    ['汪', dict(style=NORMAL), ['wang']],
    ['汪', dict(style=FINALS), ['uang']],
    ['汪', dict(style=FINALS, strict=False), ['ang']],
    ['翁', dict(style=NORMAL), ['weng']],
    ['翁', dict(style=FINALS), ['ueng']],
    ['翁', dict(style=FINALS, strict=False), ['eng']],

    ['武', dict(style=NORMAL), ['wu']],
    ['武', dict(style=NORMAL, strict=False), ['wu']],
    ['武', dict(style=TONE), ['wǔ']],
    ['武', dict(style=TONE, strict=False), ['wǔ']],
    ['武', dict(style=TONE2), ['wu3']],
    ['武', dict(style=TONE2, strict=False), ['wu3']],
    ['武', dict(style=TONE3), ['wu3']],
    ['武', dict(style=TONE3, strict=False), ['wu3']],
    ['武', dict(style=INITIALS), ['']],
    ['武', dict(style=INITIALS, strict=False), ['w']],
    ['武', dict(style=FIRST_LETTER), ['w']],
    ['武', dict(style=FIRST_LETTER, strict=False), ['w']],
    ['武', dict(style=FINALS), ['u']],
    ['武', dict(style=FINALS, strict=False), ['u']],
    ['武', dict(style=FINALS_TONE), ['ǔ']],
    ['武', dict(style=FINALS_TONE, strict=False), ['ǔ']],
    ['武', dict(style=FINALS_TONE2), ['u3']],
    ['武', dict(style=FINALS_TONE2, strict=False), ['u3']],
    ['武', dict(style=FINALS_TONE3), ['u3']],
    ['武', dict(style=FINALS_TONE3, strict=False), ['u3']],

    ['旺', dict(style=NORMAL), ['wang']],
    ['旺', dict(style=NORMAL, strict=False), ['wang']],
    ['旺', dict(style=TONE), ['wàng']],
    ['旺', dict(style=TONE, strict=False), ['wàng']],
    ['旺', dict(style=TONE2), ['wa4ng']],
    ['旺', dict(style=TONE2, strict=False), ['wa4ng']],
    ['旺', dict(style=TONE3), ['wang4']],
    ['旺', dict(style=TONE3, strict=False), ['wang4']],
    ['旺', dict(style=INITIALS), ['']],
    ['旺', dict(style=INITIALS, strict=False), ['w']],
    ['旺', dict(style=FIRST_LETTER), ['w']],
    ['旺', dict(style=FIRST_LETTER, strict=False), ['w']],
    ['旺', dict(style=FINALS), ['uang']],
    ['旺', dict(style=FINALS, strict=False), ['ang']],
    ['旺', dict(style=FINALS_TONE), ['uàng']],
    ['旺', dict(style=FINALS_TONE, strict=False), ['àng']],
    ['旺', dict(style=FINALS_TONE2), ['ua4ng']],
    ['旺', dict(style=FINALS_TONE2, strict=False), ['a4ng']],
    ['旺', dict(style=FINALS_TONE3), ['uang4']],
    ['旺', dict(style=FINALS_TONE3, strict=False), ['ang4']],


    # ü行的韵母，前面没有声母的时候，写成yu(迂)，yue(约)，yuan(冤)，
    ['迂', dict(style=NORMAL), ['yu']],
    ['迂', dict(style=FINALS), ['v']],
    ['迂', dict(style=FINALS, strict=False), ['u']],
    ['约', dict(style=NORMAL), ['yue']],
    ['约', dict(style=FINALS), ['ve']],
    ['约', dict(style=FINALS, strict=False), ['ue']],
    ['冤', dict(style=NORMAL), ['yuan']],
    ['冤', dict(style=FINALS), ['van']],
    ['冤', dict(style=FINALS, strict=False), ['uan']],

    ['鱼', dict(style=NORMAL), ['yu']],
    ['鱼', dict(style=NORMAL, strict=False), ['yu']],
    ['鱼', dict(style=TONE), ['yú']],
    ['鱼', dict(style=TONE, strict=False), ['yú']],
    ['鱼', dict(style=TONE2), ['yu2']],
    ['鱼', dict(style=TONE2, strict=False), ['yu2']],
    ['鱼', dict(style=TONE3), ['yu2']],
    ['鱼', dict(style=TONE3, strict=False), ['yu2']],
    ['鱼', dict(style=INITIALS), ['']],
    ['鱼', dict(style=INITIALS, strict=False), ['y']],
    ['鱼', dict(style=FIRST_LETTER), ['y']],
    ['鱼', dict(style=FIRST_LETTER, strict=False), ['y']],
    ['鱼', dict(style=FINALS), ['v']],
    ['鱼', dict(style=FINALS, strict=False), ['u']],
    ['鱼', dict(style=FINALS_TONE), ['ǘ']],
    ['鱼', dict(style=FINALS_TONE, strict=False), ['ú']],
    ['鱼', dict(style=FINALS_TONE2), ['v2']],
    ['鱼', dict(style=FINALS_TONE2, strict=False), ['u2']],
    ['鱼', dict(style=FINALS_TONE3), ['v2']],
    ['鱼', dict(style=FINALS_TONE3, strict=False), ['u2']],

    ['约', dict(style=NORMAL), ['yue']],
    ['约', dict(style=NORMAL, strict=False), ['yue']],
    ['约', dict(style=TONE), ['yuē']],
    ['约', dict(style=TONE, strict=False), ['yuē']],
    ['约', dict(style=TONE2), ['yue1']],
    ['约', dict(style=TONE2, strict=False), ['yue1']],
    ['约', dict(style=TONE3), ['yue1']],
    ['约', dict(style=TONE3, strict=False), ['yue1']],
    ['约', dict(style=INITIALS), ['']],
    ['约', dict(style=INITIALS, strict=False), ['y']],
    ['约', dict(style=FIRST_LETTER), ['y']],
    ['约', dict(style=FIRST_LETTER, strict=False), ['y']],
    ['约', dict(style=FINALS), ['ve']],
    ['约', dict(style=FINALS, strict=False), ['ue']],
    ['约', dict(style=FINALS_TONE), ['üē']],
    ['约', dict(style=FINALS_TONE, strict=False), ['uē']],
    ['约', dict(style=FINALS_TONE2), ['ve1']],
    ['约', dict(style=FINALS_TONE2, strict=False), ['ue1']],
    ['约', dict(style=FINALS_TONE3), ['ve1']],
    ['约', dict(style=FINALS_TONE3, strict=False), ['ue1']],

    ['元', dict(style=NORMAL), ['yuan']],
    ['元', dict(style=NORMAL, strict=False), ['yuan']],
    ['元', dict(style=TONE), ['yuán']],
    ['元', dict(style=TONE, strict=False), ['yuán']],
    ['元', dict(style=TONE2), ['yua2n']],
    ['元', dict(style=TONE2, strict=False), ['yua2n']],
    ['元', dict(style=TONE3), ['yuan2']],
    ['元', dict(style=TONE3, strict=False), ['yuan2']],
    ['元', dict(style=INITIALS), ['']],
    ['元', dict(style=INITIALS, strict=False), ['y']],
    ['元', dict(style=FIRST_LETTER), ['y']],
    ['元', dict(style=FIRST_LETTER, strict=False), ['y']],
    ['元', dict(style=FINALS), ['van']],
    ['元', dict(style=FINALS, strict=False), ['uan']],
    ['元', dict(style=FINALS_TONE), ['üán']],
    ['元', dict(style=FINALS_TONE, strict=False), ['uán']],
    ['元', dict(style=FINALS_TONE2), ['va2n']],
    ['元', dict(style=FINALS_TONE2, strict=False), ['ua2n']],
    ['元', dict(style=FINALS_TONE3), ['van2']],
    ['元', dict(style=FINALS_TONE3, strict=False), ['uan2']],

    # yun 不应该受 un -> uen 规则的影响
    ['晕', dict(style=NORMAL), ['yun']],
    ['晕', dict(style=NORMAL, strict=False), ['yun']],
    ['晕', dict(style=TONE), ['yūn']],
    ['晕', dict(style=TONE, strict=False), ['yūn']],
    ['晕', dict(style=TONE2), ['yu1n']],
    ['晕', dict(style=TONE2, strict=False), ['yu1n']],
    ['晕', dict(style=TONE3), ['yun1']],
    ['晕', dict(style=TONE3, strict=False), ['yun1']],
    ['晕', dict(style=INITIALS), ['']],
    ['晕', dict(style=INITIALS, strict=False), ['y']],
    ['晕', dict(style=FIRST_LETTER), ['y']],
    ['晕', dict(style=FIRST_LETTER, strict=False), ['y']],
    ['晕', dict(style=FINALS), ['vn']],
    ['晕', dict(style=FINALS, strict=False), ['un']],
    ['晕', dict(style=FINALS_TONE), ['ǖn']],
    ['晕', dict(style=FINALS_TONE, strict=False), ['ūn']],
    ['晕', dict(style=FINALS_TONE2), ['v1n']],
    ['晕', dict(style=FINALS_TONE2, strict=False), ['u1n']],
    ['晕', dict(style=FINALS_TONE3), ['vn1']],
    ['晕', dict(style=FINALS_TONE3, strict=False), ['un1']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_zero_consonant)
def test_zero_consonant(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result
    assert pinyin(hans, **kwargs) == [result]


data_for_uv = [
    # ü行的韵跟声母j，q，x拼的时候，写成ju(居)，qu(区)，xu(虚)，
    # ü上两点也省略；但是跟声母n，l拼的时候，仍然写成nü(女)，lü(吕)。
    ['居', dict(style=NORMAL), ['ju']],
    ['居', dict(style=FINALS), ['v']],
    ['居', dict(style=FINALS, strict=False), ['u']],
    ['区', dict(style=NORMAL), ['qu']],
    ['区', dict(style=FINALS), ['v']],
    ['区', dict(style=FINALS, strict=False), ['u']],
    ['虚', dict(style=NORMAL), ['xu']],
    ['虚', dict(style=FINALS), ['v']],
    ['虚', dict(style=FINALS, strict=False), ['u']],
    ['女', dict(style=NORMAL), ['nv']],
    ['女', dict(style=FINALS), ['v']],
    ['女', dict(style=FINALS, strict=False), ['v']],
    ['吕', dict(style=NORMAL), ['lv']],
    ['吕', dict(style=FINALS), ['v']],
    ['吕', dict(style=FINALS, strict=False), ['v']],

    ['具', dict(style=NORMAL), ['ju']],
    ['具', dict(style=NORMAL, strict=False), ['ju']],
    ['具', dict(style=TONE), ['jù']],
    ['具', dict(style=TONE, strict=False), ['jù']],
    ['具', dict(style=TONE2), ['ju4']],
    ['具', dict(style=TONE2, strict=False), ['ju4']],
    ['具', dict(style=TONE3), ['ju4']],
    ['具', dict(style=TONE3, strict=False), ['ju4']],
    ['具', dict(style=INITIALS), ['j']],
    ['具', dict(style=INITIALS, strict=False), ['j']],
    ['具', dict(style=FIRST_LETTER), ['j']],
    ['具', dict(style=FIRST_LETTER, strict=False), ['j']],
    ['具', dict(style=FINALS), ['v']],
    ['具', dict(style=FINALS, strict=False), ['u']],
    ['具', dict(style=FINALS_TONE), ['ǜ']],
    ['具', dict(style=FINALS_TONE, strict=False), ['ù']],
    ['具', dict(style=FINALS_TONE2), ['v4']],
    ['具', dict(style=FINALS_TONE2, strict=False), ['u4']],
    ['具', dict(style=FINALS_TONE3), ['v4']],
    ['具', dict(style=FINALS_TONE3, strict=False), ['u4']],

    ['取', dict(style=NORMAL), ['qu']],
    ['取', dict(style=NORMAL, strict=False), ['qu']],
    ['取', dict(style=TONE), ['qǔ']],
    ['取', dict(style=TONE, strict=False), ['qǔ']],
    ['取', dict(style=TONE2), ['qu3']],
    ['取', dict(style=TONE2, strict=False), ['qu3']],
    ['取', dict(style=TONE3), ['qu3']],
    ['取', dict(style=TONE3, strict=False), ['qu3']],
    ['取', dict(style=INITIALS), ['q']],
    ['取', dict(style=INITIALS, strict=False), ['q']],
    ['取', dict(style=FIRST_LETTER), ['q']],
    ['取', dict(style=FIRST_LETTER, strict=False), ['q']],
    ['取', dict(style=FINALS), ['v']],
    ['取', dict(style=FINALS, strict=False), ['u']],
    ['取', dict(style=FINALS_TONE), ['ǚ']],
    ['取', dict(style=FINALS_TONE, strict=False), ['ǔ']],
    ['取', dict(style=FINALS_TONE2), ['v3']],
    ['取', dict(style=FINALS_TONE2, strict=False), ['u3']],
    ['取', dict(style=FINALS_TONE3), ['v3']],
    ['取', dict(style=FINALS_TONE3, strict=False), ['u3']],

    ['徐', dict(style=NORMAL), ['xu']],
    ['徐', dict(style=NORMAL, strict=False), ['xu']],
    ['徐', dict(style=TONE), ['xú']],
    ['徐', dict(style=TONE, strict=False), ['xú']],
    ['徐', dict(style=TONE2), ['xu2']],
    ['徐', dict(style=TONE2, strict=False), ['xu2']],
    ['徐', dict(style=TONE3), ['xu2']],
    ['徐', dict(style=TONE3, strict=False), ['xu2']],
    ['徐', dict(style=INITIALS), ['x']],
    ['徐', dict(style=INITIALS, strict=False), ['x']],
    ['徐', dict(style=FIRST_LETTER), ['x']],
    ['徐', dict(style=FIRST_LETTER, strict=False), ['x']],
    ['徐', dict(style=FINALS), ['v']],
    ['徐', dict(style=FINALS, strict=False), ['u']],
    ['徐', dict(style=FINALS_TONE), ['ǘ']],
    ['徐', dict(style=FINALS_TONE, strict=False), ['ú']],
    ['徐', dict(style=FINALS_TONE2), ['v2']],
    ['徐', dict(style=FINALS_TONE2, strict=False), ['u2']],
    ['徐', dict(style=FINALS_TONE3), ['v2']],
    ['徐', dict(style=FINALS_TONE3, strict=False), ['u2']],

    ['女', dict(style=NORMAL), ['nv']],
    ['女', dict(style=NORMAL, strict=False), ['nv']],
    ['女', dict(style=TONE), ['nǚ']],
    ['女', dict(style=TONE, strict=False), ['nǚ']],
    ['女', dict(style=TONE2), ['nv3']],
    ['女', dict(style=TONE2, strict=False), ['nv3']],
    ['女', dict(style=TONE3), ['nv3']],
    ['女', dict(style=TONE3, strict=False), ['nv3']],
    ['女', dict(style=INITIALS), ['n']],
    ['女', dict(style=INITIALS, strict=False), ['n']],
    ['女', dict(style=FIRST_LETTER), ['n']],
    ['女', dict(style=FIRST_LETTER, strict=False), ['n']],
    ['女', dict(style=FINALS), ['v']],
    ['女', dict(style=FINALS, strict=False), ['v']],
    ['女', dict(style=FINALS_TONE), ['ǚ']],
    ['女', dict(style=FINALS_TONE, strict=False), ['ǚ']],
    ['女', dict(style=FINALS_TONE2), ['v3']],
    ['女', dict(style=FINALS_TONE2, strict=False), ['v3']],
    ['女', dict(style=FINALS_TONE3), ['v3']],
    ['女', dict(style=FINALS_TONE3, strict=False), ['v3']],

    ['吕', dict(style=NORMAL), ['lv']],
    ['吕', dict(style=NORMAL, strict=False), ['lv']],
    ['吕', dict(style=TONE), ['lǚ']],
    ['吕', dict(style=TONE, strict=False), ['lǚ']],
    ['吕', dict(style=TONE2), ['lv3']],
    ['吕', dict(style=TONE2, strict=False), ['lv3']],
    ['吕', dict(style=TONE3), ['lv3']],
    ['吕', dict(style=TONE3, strict=False), ['lv3']],
    ['吕', dict(style=INITIALS), ['l']],
    ['吕', dict(style=INITIALS, strict=False), ['l']],
    ['吕', dict(style=FIRST_LETTER), ['l']],
    ['吕', dict(style=FIRST_LETTER, strict=False), ['l']],
    ['吕', dict(style=FINALS), ['v']],
    ['吕', dict(style=FINALS, strict=False), ['v']],
    ['吕', dict(style=FINALS_TONE), ['ǚ']],
    ['吕', dict(style=FINALS_TONE, strict=False), ['ǚ']],
    ['吕', dict(style=FINALS_TONE2), ['v3']],
    ['吕', dict(style=FINALS_TONE2, strict=False), ['v3']],
    ['吕', dict(style=FINALS_TONE3), ['v3']],
    ['吕', dict(style=FINALS_TONE3, strict=False), ['v3']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_uv)
def test_uv(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result
    assert pinyin(hans, **kwargs) == [result]


data_for_iou = [
    # iou，uei，uen前面加声母的时候，写成iu，ui，un。
    # 例如niu(牛)，gui(归)，lun(论)。
    ['牛', dict(style=NORMAL), ['niu']],
    ['牛', dict(style=FINALS), ['iou']],
    ['牛', dict(style=FINALS, strict=False), ['iu']],
    ['归', dict(style=NORMAL), ['gui']],
    ['归', dict(style=FINALS), ['uei']],
    ['归', dict(style=FINALS, strict=False), ['ui']],
    ['论', dict(style=NORMAL), ['lun']],
    ['论', dict(style=FINALS), ['uen']],
    ['论', dict(style=FINALS, strict=False), ['un']],

    ['牛', dict(style=NORMAL), ['niu']],
    ['牛', dict(style=NORMAL, strict=False), ['niu']],
    ['牛', dict(style=TONE), ['niú']],
    ['牛', dict(style=TONE, strict=False), ['niú']],
    ['牛', dict(style=TONE2), ['niu2']],
    ['牛', dict(style=TONE2, strict=False), ['niu2']],
    ['牛', dict(style=TONE3), ['niu2']],
    ['牛', dict(style=TONE3, strict=False), ['niu2']],
    ['牛', dict(style=INITIALS), ['n']],
    ['牛', dict(style=INITIALS, strict=False), ['n']],
    ['牛', dict(style=FIRST_LETTER), ['n']],
    ['牛', dict(style=FIRST_LETTER, strict=False), ['n']],
    ['牛', dict(style=FINALS), ['iou']],
    ['牛', dict(style=FINALS, strict=False), ['iu']],
    ['牛', dict(style=FINALS_TONE), ['ioú']],
    ['牛', dict(style=FINALS_TONE, strict=False), ['iú']],
    ['牛', dict(style=FINALS_TONE2), ['iou2']],
    ['牛', dict(style=FINALS_TONE2, strict=False), ['iu2']],
    ['牛', dict(style=FINALS_TONE3), ['iou2']],
    ['牛', dict(style=FINALS_TONE3, strict=False), ['iu2']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_iou)
def test_iou(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result
    assert pinyin(hans, **kwargs) == [result]


data_for_uei = [
    # iou，uei，uen前面加声母的时候，写成iu，ui，un。
    # 例如niu(牛)，gui(归)，lun(论)。
    ['鬼', dict(style=NORMAL), ['gui']],
    ['鬼', dict(style=NORMAL, strict=False), ['gui']],
    ['鬼', dict(style=TONE), ['guǐ']],
    ['鬼', dict(style=TONE, strict=False), ['guǐ']],
    ['鬼', dict(style=TONE2), ['gui3']],
    ['鬼', dict(style=TONE2, strict=False), ['gui3']],
    ['鬼', dict(style=TONE3), ['gui3']],
    ['鬼', dict(style=TONE3, strict=False), ['gui3']],
    ['鬼', dict(style=INITIALS), ['g']],
    ['鬼', dict(style=INITIALS, strict=False), ['g']],
    ['鬼', dict(style=FIRST_LETTER), ['g']],
    ['鬼', dict(style=FIRST_LETTER, strict=False), ['g']],
    ['鬼', dict(style=FINALS), ['uei']],
    ['鬼', dict(style=FINALS, strict=False), ['ui']],
    ['鬼', dict(style=FINALS_TONE), ['ueǐ']],
    ['鬼', dict(style=FINALS_TONE, strict=False), ['uǐ']],
    ['鬼', dict(style=FINALS_TONE2), ['uei3']],
    ['鬼', dict(style=FINALS_TONE2, strict=False), ['ui3']],
    ['鬼', dict(style=FINALS_TONE3), ['uei3']],
    ['鬼', dict(style=FINALS_TONE3, strict=False), ['ui3']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_uei)
def test_uei(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result
    assert pinyin(hans, **kwargs) == [result]


data_for_uen = [
    # iou，uei，uen前面加声母的时候，写成iu，ui，un。
    # 例如niu(牛)，gui(归)，lun(论)。
    ['论', dict(style=NORMAL), ['lun']],
    ['论', dict(style=TONE), ['lùn']],
    ['论', dict(style=TONE, strict=False), ['lùn']],
    ['论', dict(style=TONE2), ['lu4n']],
    ['论', dict(style=TONE2, strict=False), ['lu4n']],
    ['论', dict(style=TONE3), ['lun4']],
    ['论', dict(style=TONE3, strict=False), ['lun4']],
    ['论', dict(style=INITIALS), ['l']],
    ['论', dict(style=INITIALS, strict=False), ['l']],
    ['论', dict(style=FIRST_LETTER), ['l']],
    ['论', dict(style=FIRST_LETTER, strict=False), ['l']],
    ['论', dict(style=FINALS), ['uen']],
    ['论', dict(style=FINALS, strict=False), ['un']],
    ['论', dict(style=FINALS_TONE), ['ùen']],
    ['论', dict(style=FINALS_TONE, strict=False), ['ùn']],
    ['论', dict(style=FINALS_TONE2), ['u4en']],
    ['论', dict(style=FINALS_TONE2, strict=False), ['u4n']],
    ['论', dict(style=FINALS_TONE3), ['uen4']],
    ['论', dict(style=FINALS_TONE3, strict=False), ['un4']],
]


@pytest.mark.parametrize('hans, kwargs, result', data_for_uen)
def test_uen(hans, kwargs, result):
    assert lazy_pinyin(hans, **kwargs) == result
    assert pinyin(hans, **kwargs) == [result]


# 所有汉字的韵母结果都是汉语拼音方案中的韵母
@pytest.mark.skipif(not SUPPORT_UCS4, reason='dont support ucs4')
def test_ensure_finails_in_strict_mode():
    for c in pinyin_dict.pinyin_dict:
        if PY2:
            han = unichr(c)  # noqa
        else:
            han = chr(c)
        raw_pinyin = pinyin(han, heteronym=True)
        pinyin_list = pinyin(han, style=Style.FINALS, strict=True,
                             heteronym=True, v_to_u=True)[0]
        for i, ret in enumerate(pinyin_list):
            if ret:
                assert ret in _FINALS, (han, raw_pinyin[i], ret)


if __name__ == '__main__':
    import pytest
    pytest.cmdline.main()
